<?php

/**
 * Expose menu items as context reactions.
 */
class context_reaction_alter_form extends context_reaction {
  /**
   * Provide a form elements allowing adding fields and values.
   */
  function options_form($context) {

    // Get current values.
    $values = $this->fetch_from_context($context);
    
    // Build existing entries.
    $form = array();
    $val_id = -1;
    foreach ($values as $val_id => $val) {
        $form[$val_id] = array(
            '#type' => 'fieldset',
            '#title' => t('Element') . ' ' . $val_id,
            '#title_display' => 'invisible',
            '#attributes' => array('class' => array('container-inline')), 
        );
        $form[$val_id]['form_id'] = array(
            '#title' => t('Form:'),
            '#default_value' => $val['form_id'],
            '#type' => 'textfield',
            '#size' => 30,
        );
        $form[$val_id]['parents'] = array(
            '#title' => t('Parents:'),
            '#default_value' => $val['parents'],
            '#type' => 'textfield',
            '#size' => 30,
        );
        $form[$val_id]['attribute'] = array(
            '#title' => t('Attribute:'),
            '#default_value' => $val['attribute'],
            '#type' => 'textfield',
            '#size' => 20,
        );
        $form[$val_id]['value'] = array(
            '#title' => t('Value:'),
            '#default_value' => $val['value'],
            '#type' => 'textfield',
            '#size' => 20,
        );
    }

    // Add space for a new addition.
    // @todo Allow adding more. BEWARE: There is no $form_state
    // http://drupal.stackexchange.com/questions/32152/add-another-item-for-custom-forms
    //http://drupalcode.org/project/flush_page_cache.git/blob/refs/heads/7.x-1.x:/flush_page_cache.admin.inc
    $form[$val_id + 1] = array(
      '#type' => 'fieldset',
      '#title' => t('New Element'),
      '#description' => t('Only one at a time per save.'),
      //'#collapsible' => FALSE, 
      //'#collapsed' => FALSE,
    );
    $form[$val_id + 1]['form_id'] = array(
        '#title' => t('Form ID'),
        '#type' => 'textfield',
        '#description' => t('The form that holds the element.'),
        '#size' => 25, 
    );
    $form[$val_id + 1]['parents'] = array(
        '#title' => t('Parent Heirarchy'),
        '#type' => 'textfield',
        '#description' => t('Parent listing including the element (comma separated), eg: "group_THIS, field_THIS"'),
        '#size' => 25,
    );
    $form[$val_id + 1]['attribute'] = array(
        '#title' => t('Attribute'),
        '#type' => 'textfield',
        '#description' => t('What attribute to alter, eg: #default_value, #label.'),
        '#size' => 25, 
    );
    $form[$val_id + 1]['value'] = array(
        '#title' => t('Default Value'),
        '#type' => 'textfield',
        '#description' => t('Set the form element to this #default_value.'),
        '#size' => 25, 
    );

    return $form;
  }

  /**
   * Override of options_form_submit().
   * Clean up empty values for storage in neat, tidy array.
   */
  function options_form_submit($values) {
    foreach ($values as $key => $option)  {
        if($option['form_id'] == '' || $option['parents'] == '' || $option['attribute'] == '') {
            unset($values[$key]);
        }
    }
    return $values;
  }

  function execute(&$form) {
    $contexts = $this->get_contexts();
    foreach ($contexts as $context) {
      if (!empty($context->reactions[$this->plugin])) {
        foreach ($context->reactions[$this->plugin] as $option_id => $option) {

            // Whether we should bother.
            if ($option['form_id'] === $form['#form_id']) {

                // Alter the form value.
                // @todo Add option for forcing.
                $parents = array();
                $parents = explode(',', $option['parents']);
                $parents[] = $option['attribute'];
                // Trim up the values, allowing messier/easier user input.
                array_walk($parents, create_function('&$val', '$val = trim($val);'));
                drupal_array_set_nested_value($form, $parents, $option['value']);  

            }

        }
      }
    }
  }
  
}
